"""Utils"""
from dataclasses import dataclass
from typing import Any, Dict, Tuple

from llamazure.azrest.models import ensure
from llamazure.rid import rid


@dataclass
class JSONTraverser:
	"""Traverse a JSON structure and replace exact string matches"""

	replacements: Dict[str, str]

	def traverse(self, obj: Any) -> Any:
		"""Traverse a JSON structure and replace exact string matches"""
		if isinstance(obj, dict):
			return {key: self.traverse(value) for key, value in obj.items()}
		elif isinstance(obj, list):
			return [self.traverse(item) for item in obj]
		elif isinstance(obj, str):
			return self.replacements.get(obj, obj)
		else:
			return obj


def rid_params(res: rid.Resource) -> Tuple[str, str, str]:
	return ensure(res.sub).uuid, ensure(res.rg).name, res.name



# pylint: disable
# flake8: noqa
from __future__ import annotations
from enum import Enum
from typing import Annotated, List, Optional, Union

from pydantic import BaseModel, Field

from llamazure.azrest.models import AzList, ReadOnly, Req, default_list, default_dict



class Resource(BaseModel):
	"""Common fields that are returned in the response for all Azure Resource Manager resources"""

	rid: ReadOnly[str] = Field(alias="id", default=None)
	name: ReadOnly[str] = None
	type: ReadOnly[str] = None

	def __eq__(self, o) -> bool:
		return (
			isinstance(o, self.__class__)
		)



class AzureEntityResource(BaseModel):
	"""The resource model definition for an Azure Resource Manager resource with an etag."""

	etag: ReadOnly[str] = None
	rid: ReadOnly[str] = Field(alias="id", default=None)
	name: ReadOnly[str] = None
	type: ReadOnly[str] = None

	def __eq__(self, o) -> bool:
		return (
			isinstance(o, self.__class__)
		)



class TrackedResource(BaseModel):
	"""The resource model definition for an Azure Resource Manager tracked top level resource which has 'tags' and a 'location'"""

	tags: Optional[dict] = None
	location: str
	rid: ReadOnly[str] = Field(alias="id", default=None)
	name: ReadOnly[str] = None
	type: ReadOnly[str] = None

	def __eq__(self, o) -> bool:
		return (
			isinstance(o, self.__class__)
			and self.tags == o.tags
			and self.location == o.location
		)



class ResourceModelWithAllowedPropertySet(BaseModel):
	"""The resource model definition containing the full set of allowed properties for a resource. Except properties bag, there cannot be a top level property outside of this set."""

	rid: ReadOnly[str] = Field(alias="id", default=None)
	name: ReadOnly[str] = None
	type: ReadOnly[str] = None
	location: Optional[str] = None
	managedBy: Optional[str] = None
	kind: Optional[str] = None
	etag: ReadOnly[str] = None
	tags: Optional[dict] = None
	identity: Optional[Identity] = None
	sku: Optional[Sku] = None
	plan: Optional[Plan] = None

	def __eq__(self, o) -> bool:
		return (
			isinstance(o, self.__class__)
			and self.location == o.location
			and self.managedBy == o.managedBy
			and self.kind == o.kind
			and self.tags == o.tags
			and self.identity == o.identity
			and self.sku == o.sku
			and self.plan == o.plan
		)



class Sku(BaseModel):
	"""The resource model definition representing SKU"""

	name: str
	tier: Optional[str] = None
	size: Optional[str] = None
	family: Optional[str] = None
	capacity: Optional[int] = None

	def __eq__(self, o) -> bool:
		return (
			isinstance(o, self.__class__)
			and self.name == o.name
			and self.tier == o.tier
			and self.size == o.size
			and self.family == o.family
			and self.capacity == o.capacity
		)



class Identity(BaseModel):
	"""Identity for the resource."""

	principalId: ReadOnly[str] = None
	tenantId: ReadOnly[str] = None
	type: Optional[str] = None

	def __eq__(self, o) -> bool:
		return (
			isinstance(o, self.__class__)
			and self.type == o.type
		)



class Plan(BaseModel):
	"""Plan for the resource."""

	name: str
	publisher: str
	product: str
	promotionCode: Optional[str] = None
	version: Optional[str] = None

	def __eq__(self, o) -> bool:
		return (
			isinstance(o, self.__class__)
			and self.name == o.name
			and self.publisher == o.publisher
			and self.product == o.product
			and self.promotionCode == o.promotionCode
			and self.version == o.version
		)



class ErrorResponse(BaseModel):
	"""Common error response for all Azure Resource Manager APIs to return error details for failed operations. (This also follows the OData error response format.)"""

	code: ReadOnly[str] = None
	message: ReadOnly[str] = None
	target: ReadOnly[str] = None
	details: Annotated[List[ErrorResponse],default_list] = []
	additionalInfo: Annotated[List[ErrorAdditionalInfo],default_list] = []

	def __eq__(self, o) -> bool:
		return (
			isinstance(o, self.__class__)
			and self.details == o.details
			and self.additionalInfo == o.additionalInfo
		)



class ErrorAdditionalInfo(BaseModel):
	"""The resource management error additional info."""

	type: ReadOnly[str] = None
	info: ReadOnly[dict] = None

	def __eq__(self, o) -> bool:
		return (
			isinstance(o, self.__class__)
		)



class Operation(BaseModel):
	"""Details of a REST API operation, returned from the Resource Provider Operations API"""

	name: ReadOnly[str] = None
	isDataAction: ReadOnly[bool] = None
	display: Optional[dict] = None
	origin: ReadOnly[str] = None
	actionType: ReadOnly[str] = None

	def __eq__(self, o) -> bool:
		return (
			isinstance(o, self.__class__)
			and self.display == o.display
		)



class LocationData(BaseModel):
	"""Metadata pertaining to the geographic location of the resource."""

	name: str
	city: Optional[str] = None
	district: Optional[str] = None
	countryOrRegion: Optional[str] = None

	def __eq__(self, o) -> bool:
		return (
			isinstance(o, self.__class__)
			and self.name == o.name
			and self.city == o.city
			and self.district == o.district
			and self.countryOrRegion == o.countryOrRegion
		)



class SystemData(BaseModel):
	"""Metadata pertaining to creation and last modification of the resource."""

	createdBy: Optional[str] = None
	createdByType: Optional[str] = None
	createdAt: Optional[str] = None
	lastModifiedBy: Optional[str] = None
	lastModifiedByType: Optional[str] = None
	lastModifiedAt: Optional[str] = None

	def __eq__(self, o) -> bool:
		return (
			isinstance(o, self.__class__)
			and self.createdBy == o.createdBy
			and self.createdByType == o.createdByType
			and self.createdAt == o.createdAt
			and self.lastModifiedBy == o.lastModifiedBy
			and self.lastModifiedByType == o.lastModifiedByType
			and self.lastModifiedAt == o.lastModifiedAt
		)



OperationListResult = AzList[Operation]

Resource.model_rebuild()

AzureEntityResource.model_rebuild()

TrackedResource.model_rebuild()

ResourceModelWithAllowedPropertySet.model_rebuild()

Sku.model_rebuild()

Identity.model_rebuild()

Plan.model_rebuild()

ErrorResponse.model_rebuild()

ErrorAdditionalInfo.model_rebuild()

Operation.model_rebuild()

LocationData.model_rebuild()

SystemData.model_rebuild()

OperationListResult.model_rebuild()





import click

from llamazure.tools.migrate import dashboard, workbook


@click.group()
def migrate():
	pass


migrate.add_command(dashboard.migrate, name="dashboard")
migrate.add_command(dashboard.restore, name="dashboard-restore")
migrate.add_command(workbook.migrate, name="workbook")
migrate.add_command(workbook.restore, name="workbook-restore")

if __name__ == "__main__":
	migrate()



"""Migrate an Azure Dashboard to a different Log Analytics Workspace"""
import json
from dataclasses import dataclass
from datetime import datetime
from pathlib import Path

import click
from azure.identity import DefaultAzureCredential

from llamazure.azrest.azrest import AzRest
from llamazure.azrest.models import cast_as
from llamazure.rid import rid
from llamazure.rid.rid import Resource
from llamazure.tools.migrate.portal.r.m.portal.portal import AzDashboards, Dashboard, PatchableDashboard  # pylint: disable=E0611,E0401
from llamazure.tools.migrate.util import JSONTraverser, rid_params


@dataclass
class Migrator:
	"""Migrate an Azure Dashboard"""

	az: AzRest
	dashboard: Resource
	transformer: JSONTraverser
	backup_directory: Path

	def migrate(self):
		"""
		Perform the migration: get the dashboard, make a backup, transform it, and update it.
		"""
		dashboard = self.get_dashboard()
		self.make_backup(dashboard)
		transformed = self.transform(dashboard)
		self.put_dashboard(transformed)

	def get_dashboard(self) -> dict:
		"""Retrieve the current dashboard data from Azure."""
		return self.az.call(AzDashboards.Get(*rid_params(self.dashboard)).with_ret_t(dict))

	def transform(self, dashboard: dict) -> dict:
		"""Transform the dashboard data using the provided transformer."""
		return self.transformer.traverse(dashboard)

	def put_dashboard(self, transformed: dict):
		"""Update the dashboard in Azure with the transformed data."""
		d = Dashboard(**transformed)
		p = cast_as(d, PatchableDashboard)
		self.az.call(
			AzDashboards.Update(*rid_params(self.dashboard), p),
		)

	def make_backup(self, dashboard: dict):
		"""Create a backup of the current dashboard data."""
		filename = self.backup_directory / Path(self.dashboard.name + datetime.utcnow().isoformat()).with_suffix(".json")
		with open(filename, "w", encoding="utf-8") as f:
			json.dump(dashboard, f)


@dataclass
class Restorer:
	az: AzRest
	dashboard: Resource
	backup: Path

	def restore(self):
		content = self.load_backup()
		self.put_dashboard(content)

	def load_backup(self):
		with open(self.backup, "r", encoding="utf-8") as f:
			return json.load(f)

	def put_dashboard(self, content: dict):
		p = PatchableDashboard(**content)
		self.az.call(
			AzDashboards.Update(*rid_params(self.dashboard), p),
		)


@click.command()
@click.option("--resource-id", help="The ID of the dashboard to migrate.")
@click.option("--replacements", help="A JSON string of the replacements to apply.")
@click.option("--backup-directory", type=click.Path(), help="The directory where backups will be stored.")
def migrate(resource_id: str, replacements: str, backup_directory: str):
	"""Migrate an Azure Dashboard to a different Log Analytics Workspace"""
	az = AzRest.from_credential(DefaultAzureCredential())

	replacements = json.loads(replacements)
	assert isinstance(replacements, dict)
	resource = rid.parse(resource_id)
	assert isinstance(resource, rid.Resource)
	transformer = JSONTraverser(replacements)
	migrator = Migrator(az, resource, transformer, Path(backup_directory))

	migrator.migrate()


@click.command()
@click.option("--resource-id", help="The ID of the dashboard to restore.")
@click.option("--backup", type=click.Path(exists=True, file_okay=True, dir_okay=False), help="The backup of the dashboard to restore")
def restore(resource_id: str, backup: str):
	"""Restore the dashboard data."""
	az = AzRest.from_credential(DefaultAzureCredential())
	resource = rid.parse(resource_id)
	assert isinstance(resource, rid.Resource)
	restorer = Restorer(az, resource, Path(backup))

	restorer.restore()


if __name__ == "__main__":
	migrate()  # pylint: disable=no-value-for-parameter



# pylint: disable
# flake8: noqa
from __future__ import annotations
from enum import Enum
from typing import Annotated, List, Optional, Union

from pydantic import BaseModel, Field

from llamazure.azrest.models import AzList, ReadOnly, Req, default_list, default_dict


from llamazure.tools.migrate.c.r.v3.managedidentity import ManagedServiceIdentity, UserAssignedIdentities
from llamazure.tools.migrate.c.r.v1.types import SystemData
class WorkbookResource(BaseModel):
	"""An azure resource object"""

	identity: Optional[ManagedServiceIdentity] = None
	kind: Optional[str] = None
	etag: Optional[str] = None
	tags: Optional[dict] = None
	location: str
	rid: ReadOnly[str] = Field(alias="id", default=None)
	name: ReadOnly[str] = None
	type: ReadOnly[str] = None

	def __eq__(self, o) -> bool:
		return (
			isinstance(o, self.__class__)
			and self.identity == o.identity
			and self.kind == o.kind
			and self.etag == o.etag
			and self.tags == o.tags
			and self.location == o.location
		)



class Workbook(BaseModel):
	"""A workbook definition."""
	class Properties(BaseModel):
		"""Properties that contain a workbook."""

		displayName: str
		serializedData: str
		version: Optional[str] = None
		timeModified: ReadOnly[str] = None
		category: str
		tags: Annotated[List[str],default_list] = []
		userId: ReadOnly[str] = None
		sourceId: Optional[str] = None
		storageUri: Optional[str] = None
		description: Optional[str] = None
		revision: ReadOnly[str] = None

		def __eq__(self, o) -> bool:
			return (
				isinstance(o, self.__class__)
				and self.displayName == o.displayName
				and self.serializedData == o.serializedData
				and self.version == o.version
				and self.category == o.category
				and self.tags == o.tags
				and self.sourceId == o.sourceId
				and self.storageUri == o.storageUri
				and self.description == o.description
			)


	properties: Properties
	systemData: Optional[SystemData] = None
	identity: Optional[ManagedServiceIdentity] = None
	kind: Optional[str] = None
	etag: Optional[str] = None
	tags: Optional[dict] = None
	location: str
	rid: ReadOnly[str] = Field(alias="id", default=None)
	name: ReadOnly[str] = None
	type: ReadOnly[str] = None

	def __eq__(self, o) -> bool:
		return (
			isinstance(o, self.__class__)
			and self.properties == o.properties
			and self.systemData == o.systemData
			and self.identity == o.identity
			and self.kind == o.kind
			and self.etag == o.etag
			and self.tags == o.tags
			and self.location == o.location
		)



class WorkbookUpdateParameters(BaseModel):
	"""The parameters that can be provided when updating workbook properties properties."""
	class Properties(BaseModel):
		"""Properties that contain a workbook for PATCH operation."""

		displayName: Optional[str] = None
		serializedData: Optional[str] = None
		category: Optional[str] = None
		tags: Annotated[List[str],default_list] = []
		description: Optional[str] = None
		revision: Optional[str] = None

		def __eq__(self, o) -> bool:
			return (
				isinstance(o, self.__class__)
				and self.displayName == o.displayName
				and self.serializedData == o.serializedData
				and self.category == o.category
				and self.tags == o.tags
				and self.description == o.description
				and self.revision == o.revision
			)


	kind: Optional[str] = None
	tags: Optional[dict] = None
	properties: Properties

	def __eq__(self, o) -> bool:
		return (
			isinstance(o, self.__class__)
			and self.kind == o.kind
			and self.tags == o.tags
			and self.properties == o.properties
		)



class WorkbookError(BaseModel):
	"""Error response."""

	error: Optional[WorkbookErrorDefinition] = None

	def __eq__(self, o) -> bool:
		return (
			isinstance(o, self.__class__)
			and self.error == o.error
		)



class WorkbookErrorDefinition(BaseModel):
	"""Error definition."""

	code: ReadOnly[str] = None
	message: ReadOnly[str] = None
	innererror: Optional[WorkbookInnerErrorTrace] = None

	def __eq__(self, o) -> bool:
		return (
			isinstance(o, self.__class__)
			and self.innererror == o.innererror
		)



class WorkbookInnerErrorTrace(BaseModel):
	"""Error details"""

	trace: Annotated[List[str],default_list] = []

	def __eq__(self, o) -> bool:
		return (
			isinstance(o, self.__class__)
			and self.trace == o.trace
		)



WorkbooksListResult = AzList[Workbook]

WorkbookResource.model_rebuild()

Workbook.model_rebuild()

WorkbookUpdateParameters.model_rebuild()

WorkbookError.model_rebuild()

WorkbookErrorDefinition.model_rebuild()

WorkbookInnerErrorTrace.model_rebuild()

WorkbooksListResult.model_rebuild()


class AzWorkbooks:
	apiv = "2023-06-01"
	@staticmethod
	def ListBySubscription(subscriptionId: str, category: str, tags: Optional[List[str]] = None, canFetchContent: Optional[bool] = None) -> Req[WorkbooksListResult]:
		"""Get all Workbooks defined within a specified subscription and category."""
		r = Req.get(
			name="Workbooks.ListBySubscription",
			path=f"/subscriptions/{subscriptionId}/providers/Microsoft.Insights/workbooks",
			apiv="2023-06-01",
			ret_t=WorkbooksListResult
		)
		if category is not None:
			r = r.add_param("category", str(category))
		if tags is not None:
			r = r.add_param("tags", str(tags))
		if canFetchContent is not None:
			r = r.add_param("canFetchContent", str(canFetchContent))

		return r

	@staticmethod
	def ListByResourceGroup(subscriptionId: str, resourceGroupName: str, category: str, tags: Optional[List[str]] = None, sourceId: Optional[str] = None, canFetchContent: Optional[bool] = None) -> Req[WorkbooksListResult]:
		"""Get all Workbooks defined within a specified resource group and category."""
		r = Req.get(
			name="Workbooks.ListByResourceGroup",
			path=f"/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Insights/workbooks",
			apiv="2023-06-01",
			ret_t=WorkbooksListResult
		)
		if category is not None:
			r = r.add_param("category", str(category))
		if tags is not None:
			r = r.add_param("tags", str(tags))
		if sourceId is not None:
			r = r.add_param("sourceId", str(sourceId))
		if canFetchContent is not None:
			r = r.add_param("canFetchContent", str(canFetchContent))

		return r

	@staticmethod
	def Get(subscriptionId: str, resourceGroupName: str, resourceName: str, canFetchContent: Optional[bool] = None) -> Req[Workbook]:
		"""Get a single workbook by its resourceName."""
		r = Req.get(
			name="Workbooks.Get",
			path=f"/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Insights/workbooks/{resourceName}",
			apiv="2023-06-01",
			ret_t=Workbook
		)
		if canFetchContent is not None:
			r = r.add_param("canFetchContent", str(canFetchContent))

		return r

	@staticmethod
	def CreateOrUpdate(subscriptionId: str, resourceGroupName: str, resourceName: str, workbookProperties: Workbook, sourceId: Optional[str] = None) -> Req[Workbook]:
		"""Create a new workbook."""
		r = Req.put(
			name="Workbooks.CreateOrUpdate",
			path=f"/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Insights/workbooks/{resourceName}",
			apiv="2023-06-01",
			body=workbookProperties,
			ret_t=Workbook
		)
		if sourceId is not None:
			r = r.add_param("sourceId", str(sourceId))

		return r

	@staticmethod
	def Delete(subscriptionId: str, resourceGroupName: str, resourceName: str) -> Req[None]:
		"""Delete a workbook."""
		r = Req.delete(
			name="Workbooks.Delete",
			path=f"/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Insights/workbooks/{resourceName}",
			apiv="2023-06-01",
			ret_t=None
		)

		return r

	@staticmethod
	def Update(subscriptionId: str, resourceGroupName: str, resourceName: str, WorkbookUpdateParameters: Optional[WorkbookUpdateParameters], sourceId: Optional[str] = None) -> Req[Workbook]:
		"""Updates a workbook that has already been added."""
		r = Req.patch(
			name="Workbooks.Update",
			path=f"/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Insights/workbooks/{resourceName}",
			apiv="2023-06-01",
			body=WorkbookUpdateParameters,
			ret_t=Workbook
		)
		if sourceId is not None:
			r = r.add_param("sourceId", str(sourceId))

		return r

	@staticmethod
	def RevisionsList(subscriptionId: str, resourceGroupName: str, resourceName: str) -> Req[WorkbooksListResult]:
		"""Get the revisions for the workbook defined by its resourceName."""
		r = Req.get(
			name="Workbooks.RevisionsList",
			path=f"/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Insights/workbooks/{resourceName}/revisions",
			apiv="2023-06-01",
			ret_t=WorkbooksListResult
		)

		return r

	@staticmethod
	def RevisionGet(subscriptionId: str, resourceGroupName: str, resourceName: str, revisionId: str) -> Req[Workbook]:
		"""Get a single workbook revision defined by its revisionId."""
		r = Req.get(
			name="Workbooks.RevisionGet",
			path=f"/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Insights/workbooks/{resourceName}/revisions/{revisionId}",
			apiv="2023-06-01",
			ret_t=Workbook
		)

		return r




# pylint: disable
# flake8: noqa
from __future__ import annotations
from enum import Enum
from typing import Annotated, List, Optional, Union

from pydantic import BaseModel, Field

from llamazure.azrest.models import AzList, ReadOnly, Req, default_list, default_dict



class UserAssignedIdentities(BaseModel):
	"""The set of user assigned identities associated with the resource. The userAssignedIdentities dictionary keys will be ARM resource ids in the form: '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}. The dictionary values can be empty objects ({}) in requests."""



	def __eq__(self, o) -> bool:
		return (
			isinstance(o, self.__class__)
		)



class UserAssignedIdentity(BaseModel):
	"""User assigned identity properties"""

	principalId: ReadOnly[str] = None
	clientId: ReadOnly[str] = None

	def __eq__(self, o) -> bool:
		return (
			isinstance(o, self.__class__)
		)



class ManagedServiceIdentity(BaseModel):
	"""Managed service identity (system assigned and/or user assigned identities)"""

	principalId: ReadOnly[str] = None
	tenantId: ReadOnly[str] = None
	type: Optional[ManagedServiceIdentityType] = None
	userAssignedIdentities: Optional[UserAssignedIdentities] = None

	def __eq__(self, o) -> bool:
		return (
			isinstance(o, self.__class__)
			and self.type == o.type
			and self.userAssignedIdentities == o.userAssignedIdentities
		)



class SystemAssignedServiceIdentity(BaseModel):
	"""Managed service identity (either system assigned, or none)"""

	principalId: ReadOnly[str] = None
	tenantId: ReadOnly[str] = None
	type: Optional[SystemAssignedServiceIdentityType] = None

	def __eq__(self, o) -> bool:
		return (
			isinstance(o, self.__class__)
			and self.type == o.type
		)



class ManagedServiceIdentityType(Enum):
	"""Type of managed service identity (where both SystemAssigned and UserAssigned types are allowed)."""
	none = "None"
	SystemAssigned = "SystemAssigned"
	UserAssigned = "UserAssigned"
	SystemAssigned_UserAssigned = "SystemAssigned,UserAssigned"


class SystemAssignedServiceIdentityType(Enum):
	"""Type of managed service identity (either system assigned, or none)."""
	none = "None"
	SystemAssigned = "SystemAssigned"


UserAssignedIdentities.model_rebuild()

UserAssignedIdentity.model_rebuild()

ManagedServiceIdentity.model_rebuild()

SystemAssignedServiceIdentity.model_rebuild()





"""Migrate an Azure Workbook to a different Log Analytics Workspace"""
import json
from dataclasses import dataclass
from datetime import datetime
from pathlib import Path

import click
from azure.identity import DefaultAzureCredential

from llamazure.azrest.azrest import AzRest
from llamazure.azrest.models import cast_as
from llamazure.rid import rid
from llamazure.rid.rid import Resource
from llamazure.tools.migrate.applicationinsights.r.m.insights.workbooks import AzWorkbooks, Workbook, WorkbookUpdateParameters  # pylint: disable=E0611,E0401
from llamazure.tools.migrate.util import JSONTraverser, rid_params


@dataclass
class Migrator:
	"""Migrate an Azure Workbook"""

	az: AzRest
	workbook: Resource
	transformer: JSONTraverser
	backup_directory: Path

	def migrate(self):
		"""Perform the migration"""
		workbook = self.get_workbook()
		print(workbook.model_dump_json(indent=2))
		self.make_backup(workbook)
		transformed = self.transform(workbook)
		self.put_workbook(transformed)

	def get_workbook(self) -> Workbook:
		"""Retrieve the current workbook data from Azure."""
		return self.az.call(AzWorkbooks.Get(*rid_params(self.workbook), canFetchContent=True))

	def transform(self, workbook: Workbook) -> Workbook:
		"""Transform the workbook data using the provided transformer."""
		workbook.properties.serializedData = json.dumps(self.transformer.traverse(json.loads(workbook.properties.serializedData)))
		return workbook

	def put_workbook(self, transformed: Workbook):
		"""Update the dashboard in Azure with the transformed data."""
		self.az.call(
			AzWorkbooks.Update(*rid_params(self.workbook), cast_as(transformed, WorkbookUpdateParameters)),
		)

	def make_backup(self, workbook: Workbook):
		"""Create a backup of the current dashboard data."""
		filename = self.backup_directory / Path(self.workbook.name + datetime.utcnow().isoformat()).with_suffix(".json")
		with open(filename, "w") as f:
			f.write(workbook.model_dump_json(indent=2))


@dataclass
class Restorer:
	az: AzRest
	workbook: Resource
	backup: Path

	def restore(self):
		content = self.load_backup()
		self.put_dashboard(content)

	def load_backup(self):
		with open(self.backup, "r", encoding="utf-8") as f:
			return json.load(f)

	def put_dashboard(self, content: dict):
		self.az.call(
			AzWorkbooks.Update(*rid_params(self.workbook), WorkbookUpdateParameters(**content)),
		)


@click.command()
@click.option("--resource-id", help="The ID of the workbook to migrate.")
@click.option("--replacements", help="A JSON string of the replacements to apply.")
@click.option("--backup-directory", type=click.Path(), help="The directory where backups will be stored.")
def migrate(resource_id: str, replacements: str, backup_directory: str):
	"""Migrate an Azure Workbook to a different Log Analytics Workspace"""
	az = AzRest.from_credential(DefaultAzureCredential())

	replacements = json.loads(replacements)
	assert isinstance(replacements, dict)
	resource = rid.parse(resource_id)
	assert isinstance(resource, rid.Resource)
	transformer = JSONTraverser(replacements)
	migrator = Migrator(az, resource, transformer, Path(backup_directory))

	migrator.migrate()


@click.command()
@click.option("--resource-id", help="The ID of the workbook to restore.")
@click.option("--backup", type=click.Path(exists=True, file_okay=True, dir_okay=False), help="The backup of the workbook to restore")
def restore(resource_id: str, backup: str):
	"""Restore the workbook data."""
	az = AzRest.from_credential(DefaultAzureCredential())
	resource = rid.parse(resource_id)
	assert isinstance(resource, rid.Resource)
	restorer = Restorer(az, resource, Path(backup))

	restorer.restore()


if __name__ == "__main__":
	migrate()  # pylint: disable=no-value-for-parameter
